@@ -7,7 +7,11 @@ use std::io::{BufRead, BufReader, Lines, Read};
 use std::path::PathBuf;
 use std::time::Duration;
 
+#[cfg(feature = "serde1")]
+use serde::{Deserialize, Serialize};
+
 bitflags! {
+    #[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
     pub struct NFSServerCaps: u32 {
 
         const NFS_CAP_READDIRPLUS = 1;
@@ -71,6 +75,7 @@ impl super::Process {
 /// For an example, see the [mountinfo.rs](https://github.com/eminence/procfs/tree/master/examples)
 /// example in the source repo.
 #[derive(Debug, Clone)]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 pub struct MountInfo {
     /// Mount ID.  A unique ID for the mount (but may be reused after `unmount`)
     pub mnt_id: i32,
@@ -180,6 +185,7 @@ impl MountInfo {
 
 /// Optional fields used in [MountInfo]
 #[derive(Debug, Clone)]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 pub enum MountOptFields {
     /// This mount point is shared in peer group.  Each peer group has a unique ID that is
     /// automatically generated by the kernel, and all mount points in the same peer group will
@@ -211,6 +217,7 @@ pub enum MountOptFields {
 /// ```
 #[derive(Debug, Clone)]
 #[cfg_attr(test, derive(PartialEq))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 pub struct MountStat {
     /// The name of the mounted device
     pub device: Option<String>,
@@ -262,6 +269,7 @@ impl MountStat {
 // https://utcc.utoronto.ca/~cks/space/blog/linux/NFSMountstatsIndex
 #[derive(Debug, Clone)]
 #[cfg_attr(test, derive(PartialEq))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 pub struct MountNFSStatistics {
     /// The version of the NFS statistics block.  Either "1.0" or "1.1".
     pub version: String,
@@ -367,6 +375,7 @@ impl MountNFSStatistics {
 /// nfs_stat_eventcounters`.
 #[derive(Debug, Copy, Clone)]
 #[cfg_attr(test, derive(PartialEq))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 pub struct NFSEventCounter {
     pub inode_revalidate: u64,
     pub deny_try_revalidate: u64,
@@ -439,6 +448,7 @@ impl NFSEventCounter {
 /// nfs_stat_bytecounters`
 #[derive(Debug, Copy, Clone)]
 #[cfg_attr(test, derive(PartialEq))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 pub struct NFSByteCounter {
     pub normal_read: u64,
     pub normal_write: u64,
@@ -497,6 +507,7 @@ impl NFSByteCounter {
 /// (source: *include/linux/sunrpc/metrics.h* `struct rpc_iostats`)
 #[derive(Debug, Clone)]
 #[cfg_attr(test, derive(PartialEq))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 pub struct NFSOperationStat {
     /// Count of rpc operations.
     pub operations: u64,
